#       This 'make' file will generate the c-tree library and executables.
#	for use with a netbios c-tree server.
#
#       To invoke the Turbo C make with this 'make' file, one has to pass
#       at least 1 parameter - MM, and a file named "mct.mac" which includes
#       the following
#         1)    TCD   - if it is defined, use default Turbo C directory path
#            or TCDIR - if TCD is not defined, the Turbo C directory path 
#                       will be stored here
#         2)    RTREE - stores the given r-tree path, if it is not defined,
#                       use default r-tree path
#         3)    TC10  - if set to 'x', it signify Turbo C version 1.0 is used

!include "mct.mac"

ASM=masm

# MM   - Memory model (s, c, m, l); it must be specified when invoking make
!if !$d(MM)
MM=s
!endif

# TCDIR                               - Turboc C directory
!if !$d(TCDIR)
TCDIR=c:\turboc
!endif

# RTREE                               - r-tree directory
!if !$d(RTREE)
RTREE=..\..\rtree
!endif

# COMPILER                            - compiler with appropriate switches
COMPILER=$(TCDIR)\tcc -c -w-

# TCP                                 - compiler switch for Turbo C lib path
TCP=-L$(TCDIR)\lib

# P                                   - path to 'include' files
#                                     - if using Turbo C 1.0, use
#                                       following definition for P instead
!if $d(TC10)
P=-I.\ -I..\ -I$(RTREE) -I$(TCDIR) -I$(TCDIR)\include -I$(TCDIR)\include\sys
!else
P=-I.\;..\;$(RTREE);$(TCDIR);$(TCDIR)\include;$(TCDIR)\include\sys
!endif

# CM                                  - compiler memory model switch
CM=-m$(MM)

# D                                   - c-tree library & utilities directory
D=$(MM)app.net

# LN                                  - c-tree library name
LN=ctalib

# ALIB                                - c-tree library name with 'lib' extension
ALIB=$(LN).lib

# LIBRARIAN                           - object modules librarian
LIBRARIAN=$(TCDIR)\tlib

# LINK                                - linker with appropiate switch
#                                       if using Turbo C 1.0 with large memory model,
#                                       use special definition 
#                                       (exit.obj has been extracted from c0l)
!if $d(TC10) && $(mm)==l
LINK=$(TCDIR)\tlink $(TCDIR)\lib\c0$(MM) $(TCDIR)\lib\exit.obj
!else
LINK=$(TCDIR)\tlink $(TCDIR)\lib\c0$(MM)
!endif

# executables ----------------------------------------------------------------

executables: $(D)\ctexmc.exe $(D)\ctexmg.exe $(D)\ctvxmg.exe $(D)\ctixmg.exe \
             $(D)\ctpkey.exe $(D)\ctstop.exe $(D)\ctstat.exe

# sample programs and utilities ----------------------------------------------

$(D)\ctexmc.exe: $(D)\ctexmc.obj $(D)\$(ALIB)
        cd $(D)
        $(LINK) $&,$&,,$(ALIB) $(TCDIR)\lib\c$(MM)
        cd ..

$(D)\ctexmc.obj: ..\ctexmc.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctexmg.exe: $(D)\ctexmg.obj $(D)\$(ALIB)
	cd $(D)
	$(LINK) $&,$&,,$(ALIB) $(TCDIR)\lib\c$(MM)
	cd ..

$(D)\ctexmg.obj: ..\ctexmg.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctvxmg.exe: $(D)\ctvxmg.obj $(D)\$(ALIB)
	cd $(D)
	$(LINK) $&,$&,,$(ALIB) $(TCDIR)\lib\c$(MM)
	cd ..

$(D)\ctvxmg.obj: ..\ctvxmg.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctixmg.exe: $(D)\ctixmg.obj $(D)\$(ALIB)
	cd $(D)
	$(LINK) $&,$&,,$(ALIB) $(TCDIR)\lib\c$(MM)
	cd ..

$(D)\ctixmg.obj: ..\ctixmg.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctpkey.exe: $(D)\ctpkey.obj $(D)\$(ALIB)
	cd $(D)
	$(LINK) $&,$&,,$(ALIB) $(TCDIR)\lib\c$(MM)
	cd ..

$(D)\ctpkey.obj: ..\ctpkey.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctstop.exe: $(D)\ctstop.obj $(D)\$(ALIB)
	cd $(D)
	$(LINK) $&,$&,,$(ALIB) $(TCDIR)\lib\c$(MM)
	cd ..

$(D)\ctstop.obj: ..\ctstop.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctstat.exe: $(D)\ctstat.obj $(D)\$(ALIB)
	cd $(D)
	$(LINK) $&,$&,,$(ALIB) $(TCDIR)\lib\c$(MM)
	cd ..

$(D)\ctstat.obj: ..\ctstat.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

# the c-tree library --------------------------------------------

$(D)\$(ALIB) :	$(D)\ctappx.obj $(D)\ctapx2.obj $(D)\ctapx3.obj \
		$(D)\ctapx4.obj $(D)\ctcomp.obj $(D)\ctvrcu.obj \
	     	$(D)\ctclb2.obj \
	     	$(D)\ctsset.obj $(D)\ctisam.obj $(D)\ctvrec.obj \
	     	$(D)\ctism2.obj $(D)\ctestm.obj \
	     	$(D)\ctismc.obj $(D)\ctismo.obj $(D)\cticre.obj \
	     	$(D)\ctiopn.obj \
	     	$(D)\cticls.obj $(D)\ctifil.obj \
	     	$(D)\ctismu.obj $(D)\ctamsg.obj $(D)\ctaxit.obj $(D)\ctnbios.obj
	cd $(D)
	del $(LN).bak
        rename $(ALIB) $(LN).bak
	$(LIBRARIAN) $(ALIB) + ctamsg + ctaxit + ctnbios
        $(LIBRARIAN) $(ALIB) + ctappx + ctapx2 + ctsset + ctisam + ctvrec + ctvrcu + ctism2
        $(LIBRARIAN) $(ALIB) + ctismc + ctismo + cticre + ctiopn + ctcomp + cticls + ctifil
        $(LIBRARIAN) $(ALIB) + ctismu + ctapx3 + ctapx4 + ctestm + ctclb2
	cd ..

# library modules ----------------------------------------------------

$(D)\ctamsg.obj: ..\netbios\ctamsg.c ..\netbios\ctaxit.h
	$(COMPILER) -I..\netbios $(P) $(TCP) $(CM) ..\netbios\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctnbios.obj: ..\netbios\ctnbios.c
	$(COMPILER) -I..\netbios $(P) $(TCP) $(CM) ..\netbios\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctaxit.obj: ..\netbios\ctaxit.asm
	$(ASM) ..\netbios\$&/ml/I..\netbios /D$(AM);
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctappx.obj: ..\ctappx.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctapx2.obj: ..\ctapx2.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctapx3.obj: ..\ctapx3.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctapx4.obj: ..\ctapx4.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctcomp.obj: ..\ctcomp.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctvrcu.obj: ..\ctvrcu.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctestm.obj: ..\ctestm.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctclb2.obj: ..\ctclb2.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctsset.obj: ..\ctsset.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctisam.obj: ..\ctisam.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctvrec.obj: ..\ctvrec.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctism2.obj: ..\ctism2.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctismc.obj: ..\ctismc.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctismo.obj: ..\ctismo.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\cticre.obj: ..\cticre.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctiopn.obj: ..\ctiopn.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\cticls.obj: ..\cticls.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctifil.obj: ..\ctifil.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

$(D)\ctismu.obj: ..\ctismu.c
	$(COMPILER) $(P) $(TCP) $(CM) ..\$&.c
	copy $&.obj $(D)\*.*
	del $&.obj

# end
